﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Diagnostics;
using System.IO;
using System.Reflection;
using Ascon.Pilot.Bim.SDK.RengaComAdapterSample;
using Ascon.Pilot.Bim.SDK.RengaModuleSample.Properties;
using Ascon.Pilot.SDK;
using Ascon.Pilot.SDK.Menu;

namespace Ascon.Pilot.Bim.SDK.RengaModuleSample
{
    [Export(typeof(IDataPlugin))]
    public class RengaModule : IDataPlugin, IMenu<IObjectNodeContext>
    {
        private readonly IClientInfo _clientInfo;
        private readonly IPilotDialogService _dialogService;

        private const string SHOW_OBJECT_IN_RENGA_COMMAND_NAME = "miShowObjectInRengaCommand";
        private const string COPY_LINK_TO_OBJECT_IN_RENGA_COMMAND_NAME = "miCopyLinkToObjectInRengaCommand";
        private const string RENGA_ICON_NAME = "Renga_logo.svg";
        private const string MODEL_PART_IFC_APPLICATION_ID_ATTR_NAME = "IFCAPPLICATION_ID";
        private const string MODEL_PART_FILE_NAME_ATTR_NAME = "FILE_NAME";
        private const string RENGA_APP_IFC_NAME = "RENGA";

        [ImportingConstructor]
        public RengaModule(IClientInfo clientInfo, IPilotServiceProvider serviceProvider, IPilotDialogService dialogService)
        {
            _clientInfo = clientInfo;
            _dialogService = dialogService;
            serviceProvider.Register<IMenu<IObjectNodeContext>>(this);
            
            //запустим RengaProtocolHandler (приложение само зарегистрируется, если нужно)
            StartProtocolHandler();
        }

        private void StartProtocolHandler()
        {
            //считаем, что RengaProtocolHandler.exe лежит в корневой папке плагина
            var executingAssemblyFilename = Assembly.GetExecutingAssembly().Location;
            var directory = Directory.GetParent(executingAssemblyFilename);
            var exeFilename = Path.Combine(directory.FullName, "RengaProtocolHandler.exe");
            if (!File.Exists(exeFilename))
            {
                //todo log
                return;
            }

            var p = new Process
            {
                StartInfo =
                {
                    UseShellExecute = false, 
                    FileName = exeFilename, 
                    Arguments = "--init"

                }
                
            };
            p.Start();
        }

        public void Build(IMenuBuilder builder, IObjectNodeContext context)
        {
            if (context.Attributes == null)
                return;

            if (!CanProvideCommands(context.Attributes))
                return;

            var iconBytes = IconLoader.GetIcon(RENGA_ICON_NAME);

            builder.AddSeparator(builder.Count - 1);

            builder.AddItem(SHOW_OBJECT_IN_RENGA_COMMAND_NAME, builder.Count - 1)
                .WithHeader(Resources.ShowObjectInRengaCommand)
                .WithIcon(iconBytes);

            builder.AddItem(COPY_LINK_TO_OBJECT_IN_RENGA_COMMAND_NAME, builder.Count - 1)
                .WithHeader(Resources.CopyLinkToObjectInRengaCommand)
                .WithIcon(iconBytes);
        }

        public void OnMenuItemClick(string name, IObjectNodeContext context)
        {
            if (name == SHOW_OBJECT_IN_RENGA_COMMAND_NAME)
                ShowObjectInRenga(context.ObjectNodeId);
            else if (name == COPY_LINK_TO_OBJECT_IN_RENGA_COMMAND_NAME)
                UriHelper.CopyObjectUriToClipboard(context.ObjectNodeId, _clientInfo.ConnectionString);
        }

        private void ShowObjectInRenga(Guid id)
        {
            var result = Adapter.ShowObjectInRenga(id);

            if (!result)
                _dialogService.ShowBalloon(Resources.ShowObjectInRengaTitle, Resources.RengaInstanceWithSourceProjectNotFoundMessage, PilotBalloonIcon.Info);
        }

        private bool CanProvideCommands(IReadOnlyDictionary<string, object> attributes)
        {
            if (attributes.TryGetValue(MODEL_PART_IFC_APPLICATION_ID_ATTR_NAME, out var appId))
            {
                if (appId.ToString().ToUpper().Contains(RENGA_APP_IFC_NAME))
                    return true;
            }

            if (attributes.TryGetValue(MODEL_PART_FILE_NAME_ATTR_NAME, out var fileName))
            {
                if (fileName.ToString().ToUpper().Contains(RENGA_APP_IFC_NAME))
                    return true;
            }

            return false;
        }
    }
}