﻿using System;
using System.ComponentModel;
using Ascon.Pilot.Common.DataProtection;
using Ascon.Pilot.DataClasses;
using Ascon.Pilot.DataModifier;
using Ascon.Pilot.Server.Api;
using Ascon.Pilot.Server.Api.Contracts;
using BackendImpl;
using log4net;

namespace ModifierSample
{
    public class RemoteApiProvider : IConnectionLostListener
    {
        private Backend _backend;
        private IServerApi _serverApi;
        private IFileStorage _fileStorageProvider;

        public void ConnectToPilotServer(ConnectionCredentials credentials)
        {
            var httpClient = new HttpPilotClient(credentials.GetConnectionString(), credentials.GetConnectionProxy());
            httpClient.SetConnectionLostListener(this);
            // set isCheckedClientVersion to true if you need to check if the versions of the client and server match
            httpClient.Connect(false);
            
            // get apis
            var authApi = httpClient.GetAuthenticationApi();
            // if you need to handle callbacks from the server, please implement the IServerCallback interface
            _serverApi = httpClient.GetServerApi(new NullableServerCallback());
            // if you need to handle messages callbacks from the server, please implement the IMessageCallback interface
            var messageApi = httpClient.GetMessagesApi(new NullableMessagesCallback());

            var fileApi = httpClient.GetFileArchiveApi();

            // auth
            // License code for pilot-ice (100) use your license code here
            authApi.Login(credentials.DatabaseName, credentials.Username, credentials.ProtectedPassword, false, 100);
            // open database
            var databaseInfo = _serverApi.OpenDatabase();

            // create instances
            var localArchiveRootFolder = DirectoryHelper.GetTempPath();
            _fileStorageProvider = new FileStorageProvider(localArchiveRootFolder);
            var changsetUploader = new ChangesetUploader(fileApi, _fileStorageProvider, null); //Set logger if needed
            _backend = new Backend(_serverApi, databaseInfo, messageApi, changsetUploader);
        }

        public IModifier GetNewModifier()
        {
            var modifier = new Modifier(_backend);
            return modifier;
        }

        public IServerApi GetServerApi()
        {
            return _serverApi;
        }

        public IBackend GetBackend()
        {
            return _backend;
        }

        public void ConnectionLost(Exception ex = null)
        {
            // todo
            //try to reconnect if needed
        }

        public IFileStorage GetStorage()
        {
            return _fileStorageProvider;
        }
    }
}
